
--[[
    TODO:
        - create access via fubar
        - create selectable list via fubar menu
    FUTURE:
        - Sort by methods
        - Replace small bag icon with drown down list
        - Add counter field to display: filtered items / total items
        - Enable / Disable commands
        - Only start flight watcher when flag is set
--]]

ZOOKEEPER_VERSION = 0.1
TYPE_MOUNTS = "MOUNT"
TYPE_COMPANIONS = "CRITTER"

ZooKeeper = LibStub("AceAddon-3.0"):NewAddon("ZooKeeper", "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0", "AceTimer-3.0");

local L = LibStub("AceLocale-3.0"):GetLocale("ZooKeeper", true)

-- BINDINGS
BINDING_HEADER_ZOOKEEPER = L.BindingHeader
BINDING_NAME_ZOOKEEPER_RECALL_HOTKEY = L.RecallHotkey

function ZooKeeper:OnInitialize()
    local baseDB = LibStub("AceDB-3.0"):New("ZooKeeperDB")
    self.db = baseDB.global
    self:InitDB()
    self:SetupOptions()
end

function ZooKeeper:InitDB()
    local version = self.db.version
    if not version or version ~= ZOOKEEPER_VERSION then
        self.db.version = ZOOKEEPER_VERSION
    end
    if not self.db.columns then
        self.db.columns = 8
    end
    if not self.db.rows then
        self.db.rows = 10
    end
    if not self.db.lastViewed then
        self.db.lastViewed = TYPE_MOUNTS
    end
    if not self.db.buttonPadding then
        self.db.buttonPadding = 4
    end
    if not self.db.scale then
        self.db.scale = 1
    end
    if not self.db.opacity then
        self.db.opacity = 1
    end
    if not self.db.mountOrder then
        self.db.mountOrder = {}
    end
    if not self.db.companionOrder then
        self.db.companionOrder = {}
    end
end


function ZooKeeper:OnEnable()
    -- Called when the addon is enabled
    self.frame = self.Frame:New(L.Title, self.db, 'pets');

    -- register stuffs
    self:RegisterEvent("COMPANION_UPDATE", "CompanionUpdateHandler");
    self:RegisterEvent("COMPANION_LEARNED", "CompanionLearnedHandler");
    self:RegisterEvent("PLAYER_ENTERING_WORLD", "EnteringWorldHandler");
    self:RegisterEvent("PLAYER_UNGHOST", "UnghostHandler");
    self:ScheduleRepeatingTimer(self.OnUpdate, 2, self)

    self.onTaxi = nil

    -- create buttons
    if not self.button then
        self.button = CreateFrame("Button", "ZooKeeperOpenButton", PetPaperDollFrame, "ZooKeeperOpenButtonTemplate");
        self.button:ClearAllPoints();
        self.button:SetPoint("TOPRIGHT", PetPaperDollFrameCompanionFrame, "TOPRIGHT", -36, -36);
        self.button.frame = self.frame;
        self.button:SetScript("OnClick", function(self) self.frame:ToggleFrame() end);
        self:SecureHook("CharacterFrameTab_OnClick");
    end
end

function ZooKeeper:OnDisable()
    -- flight taxi catcher
    self.frame:SetScript("ON_UPDATE", nil);

    -- unregister
    self:UnregisterAllEvents();

    -- cancel timers
    self:CancelAllTimers();
end

function ZooKeeper:CompanionUpdateHandler()
    if self.frame then
        self.frame:UpdatePets()
    end
end

function ZooKeeper:CompanionLearnedHandler()
    self:Update()
end

function ZooKeeper:Update()
    if self.frame and self.frame:IsShown() then
        self.frame:UpdatePets()
    end
end

function ZooKeeper:CharacterFrameTab_OnClick (charFrame, button)
    local name = charFrame:GetName();

    if ( name == "CharacterFrameTab2" ) then
        if ( not self.frame:IsShown() ) then
            if (self.db.showWithPetPaperDoll and self.db.showWithPetPaperDoll == true) then
                self:Show()
            end
        end
    end
end

function ZooKeeper:Show()
    self.frame:Show()
end

function ZooKeeper:UnghostHandler()
    if self.db.recallOnRez then
        self:RecallLastCompanion()
    end
end

-- Called on PLAYER_ENTERING_WORLD (anytime a load screen is shown)
function ZooKeeper:EnteringWorldHandler()
    if self.db.recallOnZone then
        self:RecallLastCompanion()
    end
end

function ZooKeeper:RecallLastCompanion()
    local petID = self.db.lastCompanion
    if petID then
        local creatureID, creatureName, spellID, icon, active = GetCompanionInfo(TYPE_COMPANIONS, petID);
        if (not active) then
            CallCompanion(TYPE_COMPANIONS, petID)
         end
    end
end

-- Taxi catcher, should rewrite some other way but PLAYER_LOST_CONTROL and PLAYER_GAINED_CONTROL doesn't seem to work.
--    Could instead only set the OnUpdate timer if recallOnLanding flag is set
function ZooKeeper:OnUpdate()
    if self.db.recallOnLanding then
        local newOnTaxi = UnitOnTaxi("player")
        if self.onTaxi and not newOnTaxi then
            -- flight ended
              self:RecallLastCompanion()
        end

        self.onTaxi = newOnTaxi
    end
end

--utility function: create a widget class (from Combuctor)
function ZooKeeper:NewClass(type, parentClass)
    local class = CreateFrame(type)
    class.mt = {__index = class}

    if parentClass then
        class = setmetatable(class, {__index = parentClass})
        class.super = parentClass
    end

    function class:Bind(o)
        return setmetatable(o, self.mt)
    end

    return class
end
